@@ -34,6 +34,8 @@
 
 #include <glib.h>
 
+#include <glib/gi18n-lib.h>
+
 #include "pango/cogl-pango.h"
 
 #include "clutter-backend.h"
@@ -54,32 +56,39 @@ G_BEGIN_DECLS
 
 typedef struct _ClutterMainContext      ClutterMainContext;
 
+#define CLUTTER_PRIVATE_FLAGS(a)	 (((ClutterActor *) (a))->private_flags)
+#define CLUTTER_SET_PRIVATE_FLAGS(a,f)	 (CLUTTER_PRIVATE_FLAGS (a) |= (f))
+#define CLUTTER_UNSET_PRIVATE_FLAGS(a,f) (CLUTTER_PRIVATE_FLAGS (a) &= ~(f))
+
+#define CLUTTER_ACTOR_IS_TOPLEVEL(a)            ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_IS_TOPLEVEL) != FALSE)
+#define CLUTTER_ACTOR_IS_INTERNAL_CHILD(a)      ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_INTERNAL_CHILD) != FALSE)
+#define CLUTTER_ACTOR_IN_DESTRUCTION(a)         ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_IN_DESTRUCTION) != FALSE)
+#define CLUTTER_ACTOR_IN_REPARENT(a)            ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_IN_REPARENT) != FALSE)
+#define CLUTTER_ACTOR_IN_PAINT(a)               ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_IN_PAINT) != FALSE)
+#define CLUTTER_ACTOR_IN_RELAYOUT(a)            ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_IN_RELAYOUT) != FALSE)
+#define CLUTTER_STAGE_IN_RESIZE(a)              ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_IN_RESIZE) != FALSE)
+
 typedef enum {
-  CLUTTER_ACTOR_UNUSED_FLAG    = 0,
+  CLUTTER_ACTOR_UNUSED_FLAG = 0,
 
-  CLUTTER_ACTOR_IN_DESTRUCTION = 1 << 0,
-  CLUTTER_ACTOR_IS_TOPLEVEL    = 1 << 1,
-  CLUTTER_ACTOR_IN_REPARENT    = 1 << 2,
-
-  /* Used by the stage to indicate GL viewport / perspective etc needs
-   * (re)setting.
-   */
-  CLUTTER_ACTOR_SYNC_MATRICES  = 1 << 3,
+  CLUTTER_IN_DESTRUCTION = 1 << 0,
+  CLUTTER_IS_TOPLEVEL    = 1 << 1,
+  CLUTTER_IN_REPARENT    = 1 << 2,
 
   /* Used to avoid recursion */
-  CLUTTER_ACTOR_IN_PAINT       = 1 << 4,
+  CLUTTER_IN_PAINT       = 1 << 3,
 
   /* Used to avoid recursion */
-  CLUTTER_ACTOR_IN_RELAYOUT    = 1 << 5,
+  CLUTTER_IN_RELAYOUT    = 1 << 4,
 
   /* Used by the stage if resizing is an asynchronous operation (like on
    * X11) to delay queueing relayouts until we got a notification from the
    * event handling
    */
-  CLUTTER_STAGE_IN_RESIZE      = 1 << 6,
+  CLUTTER_IN_RESIZE      = 1 << 5,
 
   /* a flag for internal children of Containers */
-  CLUTTER_ACTOR_INTERNAL_CHILD = 1 << 7
+  CLUTTER_INTERNAL_CHILD = 1 << 6
 } ClutterPrivateFlags;
 
 struct _ClutterInputDevice
@@ -182,10 +191,6 @@ gboolean            _clutter_context_is_initialized (void);
 PangoContext *_clutter_context_create_pango_context (ClutterMainContext *self);
 PangoContext *_clutter_context_get_pango_context    (ClutterMainContext *self);
 
-#define CLUTTER_PRIVATE_FLAGS(a)	 (((ClutterActor *) (a))->private_flags)
-#define CLUTTER_SET_PRIVATE_FLAGS(a,f)	 (CLUTTER_PRIVATE_FLAGS (a) |= (f))
-#define CLUTTER_UNSET_PRIVATE_FLAGS(a,f) (CLUTTER_PRIVATE_FLAGS (a) &= ~(f))
-
 #define CLUTTER_PARAM_READABLE  \
         G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB
 #define CLUTTER_PARAM_WRITABLE  \
@@ -195,6 +200,15 @@ PangoContext *_clutter_context_get_pango_context    (ClutterMainContext *self);
 
 #define I_(str)  (g_intern_static_string ((str)))
 
+/* mark all properties under the "Property" context */
+#ifdef ENABLE_NLS
+#define P_(String) (_clutter_gettext ((String)))
+#else
+#define P_(String) (String)
+#endif
+
+G_CONST_RETURN gchar *_clutter_gettext (const gchar *str);
+
 /* device manager */
 void _clutter_device_manager_add_device     (ClutterDeviceManager *device_manager,
                                              ClutterInputDevice   *device);
@@ -229,6 +243,23 @@ void                _clutter_stage_set_window           (ClutterStage       *sta
                                                          ClutterStageWindow *stage_window);
 ClutterStageWindow *_clutter_stage_get_window           (ClutterStage       *stage);
 ClutterStageWindow *_clutter_stage_get_default_window   (void);
+void                _clutter_stage_get_projection_matrix (ClutterStage *stage,
+                                                          CoglMatrix *projection);
+
+void                _clutter_stage_dirty_projection     (ClutterStage *stage);
+void                _clutter_stage_set_viewport         (ClutterStage *stage,
+                                                         int           x,
+                                                         int           y,
+                                                         int           width,
+                                                         int           height);
+void                _clutter_stage_get_viewport         (ClutterStage *stage,
+                                                         int          *x,
+                                                         int          *y,
+                                                         int          *width,
+                                                         int          *height);
+void                _clutter_stage_dirty_viewport       (ClutterStage *stage);
+
+
 void                _clutter_stage_maybe_setup_viewport (ClutterStage       *stage);
 void                _clutter_stage_maybe_relayout       (ClutterActor       *stage);
 gboolean            _clutter_stage_needs_update         (ClutterStage       *stage);
@@ -245,6 +276,12 @@ int      _clutter_stage_get_pending_swaps      (ClutterStage *stage);
 
 gboolean _clutter_stage_has_full_redraw_queued (ClutterStage *stage);
 
+void     _clutter_stage_set_pick_buffer_valid             (ClutterStage *stage,
+                                                           gboolean      valid);
+gboolean _clutter_stage_get_pick_buffer_valid             (ClutterStage *stage);
+void     _clutter_stage_increment_picks_per_frame_counter (ClutterStage *stage);
+void     _clutter_stage_reset_picks_per_frame_counter     (ClutterStage *stage);
+guint    _clutter_stage_get_picks_per_frame_counter       (ClutterStage *stage);
 
 /* vfuncs implemented by backend */
 GType         _clutter_backend_impl_get_type  (void);
@@ -270,6 +307,12 @@ gboolean      _clutter_backend_post_parse     (ClutterBackend  *backend,
                                                GError         **error);
 void          _clutter_backend_init_events    (ClutterBackend  *backend);
 
+void          _clutter_backend_copy_event_data (ClutterBackend     *backend,
+                                                const ClutterEvent *src,
+                                                ClutterEvent       *dest);
+void          _clutter_backend_free_event_data (ClutterBackend     *backend,
+                                                ClutterEvent       *event);
+
 ClutterFeatureFlags _clutter_backend_get_features (ClutterBackend *backend);
 
 gfloat        _clutter_backend_get_units_per_em   (ClutterBackend       *backend,
@@ -302,8 +345,13 @@ gboolean _clutter_boolean_handled_accumulator (GSignalInvocationHint *ihint,
                                                const GValue          *handler_return,
                                                gpointer               dummy);
 
+ClutterActor *_clutter_actor_get_stage_internal         (ClutterActor *actor);
+
+void _clutter_actor_apply_modelview_transform           (ClutterActor *self,
+                                                         CoglMatrix *matrix);
 void _clutter_actor_apply_modelview_transform_recursive (ClutterActor *self,
-						       ClutterActor *ancestor);
+						         ClutterActor *ancestor,
+                                                         CoglMatrix *matrix);
 
 void _clutter_actor_rerealize           (ClutterActor    *self,
                                          ClutterCallback  callback,
@@ -321,9 +369,9 @@ void _clutter_actor_set_enable_paint_unmapped (ClutterActor *self,
 void _clutter_actor_set_has_pointer (ClutterActor *self,
                                      gboolean      has_pointer);
 
-void _clutter_actor_transform_and_project_box (ClutterActor          *self,
-					       const ClutterActorBox *box,
-					       ClutterVertex          verts[]);
+gboolean _clutter_actor_transform_and_project_box (ClutterActor          *self,
+                                                   const ClutterActorBox *box,
+                                                   ClutterVertex          verts[]);
 
 void _clutter_actor_queue_redraw_with_clip (ClutterActor          *self,
                                             ClutterRedrawFlags     flags,
@@ -339,8 +387,28 @@ gint32 _clutter_backend_get_units_serial (ClutterBackend *backend);
 gboolean _clutter_effect_pre_paint  (ClutterEffect *effect);
 void     _clutter_effect_post_paint (ClutterEffect *effect);
 
+void _clutter_constraint_update_allocation (ClutterConstraint *constraint,
+                                            ClutterActor      *actor,
+                                            ClutterActorBox   *allocation);
+
 GType _clutter_layout_manager_get_child_meta_type (ClutterLayoutManager *manager);
 
+void     _clutter_event_set_platform_data (ClutterEvent       *event,
+                                           gpointer            data);
+gpointer _clutter_event_get_platform_data (const ClutterEvent *event);
+
+#if GLIB_CHECK_VERSION (2, 25, 9)
+
+#define _clutter_notify_by_pspec(obj, pspec) \
+  g_object_notify_by_pspec ((obj), (pspec))
+
+#else
+
+#define _clutter_notify_by_pspec(obj, pspec) \
+  g_object_notify ((obj), (pspec)->name)
+
+#endif
+
 G_END_DECLS
 
 #endif /* _HAVE_CLUTTER_PRIVATE_H */
