@@ -167,6 +167,8 @@ static guint        repaint_upload_func = 0;
 static GList       *upload_list = NULL;
 static GStaticMutex upload_list_mutex = G_STATIC_MUTEX_INIT;
 
+static CoglMaterial *texture_template_material = NULL;
+
 static void
 texture_fbo_free_resources (ClutterTexture *texture);
 
@@ -215,10 +217,12 @@ texture_free_gl_resources (ClutterTexture *texture)
   CLUTTER_MARK();
 
   if (priv->material != COGL_INVALID_HANDLE)
-    /* We want to keep the layer so that the filter settings will
-       remain but we want to free its resources so we clear the
-       texture handle */
-    cogl_material_set_layer (priv->material, 0, COGL_INVALID_HANDLE);
+    {
+      /* We want to keep the layer so that the filter settings will
+         remain but we want to free its resources so we clear the
+         texture handle */
+      cogl_material_set_layer (priv->material, 0, COGL_INVALID_HANDLE);
+    }
 }
 
 static void
@@ -505,13 +509,27 @@ update_fbo (ClutterActor *self)
 
       /* Reapply the source's parent transformations */
       if ((source_parent = clutter_actor_get_parent (priv->fbo_source)))
-        _clutter_actor_apply_modelview_transform_recursive (source_parent,
-                                                            NULL);
+        {
+          CoglMatrix modelview;
+
+          /* NB: _clutter_actor_apply_modelview_transform_recursive
+           * will never include the transformation between stage
+           * coordinates and OpenGL window coordinates, we have to
+           * explicitly use the stage->apply_transform to get that...
+           */
+
+          cogl_matrix_init_identity (&modelview);
+          _clutter_actor_apply_modelview_transform (stage, &modelview);
+          _clutter_actor_apply_modelview_transform_recursive (source_parent,
+                                                              NULL,
+                                                              &modelview);
+          cogl_set_modelview_matrix (&modelview);
+        }
     }
 
 
   /* cogl_clear is called to clear the buffers */
-  cogl_color_set_from_4ub (&transparent_col, 0, 0, 0, 0);
+  cogl_color_init_from_4ub (&transparent_col, 0, 0, 0, 0);
   cogl_clear (&transparent_col,
               COGL_BUFFER_BIT_COLOR |
               COGL_BUFFER_BIT_DEPTH);
@@ -559,7 +577,7 @@ create_pick_material (ClutterActor *self)
 {
   ClutterTexture *texture = CLUTTER_TEXTURE (self);
   ClutterTexturePrivate *priv = texture->priv;
-  CoglHandle pick_material = cogl_material_new ();
+  CoglHandle pick_material = cogl_material_copy (texture_template_material);
   GError *error = NULL;
 
   if (!cogl_material_set_layer_combine (pick_material, 0,
@@ -615,11 +633,11 @@ clutter_texture_pick (ClutterActor       *self,
       if (priv->fbo_handle != COGL_INVALID_HANDLE)
         update_fbo (self);
 
-      cogl_color_set_from_4ub (&pick_color,
-                               color->red,
-                               color->green,
-                               color->blue,
-                               0xff);
+      cogl_color_init_from_4ub (&pick_color,
+                                color->red,
+                                color->green,
+                                color->blue,
+                                0xff);
       cogl_material_set_layer_combine_constant (priv->pick_material,
                                                 0, &pick_color);
       cogl_material_set_layer (priv->pick_material, 0,
@@ -732,21 +750,13 @@ static void
 clutter_texture_dispose (GObject *object)
 {
   ClutterTexture *texture = CLUTTER_TEXTURE (object);
+  ClutterTexturePrivate *priv = texture->priv;
 
   texture_free_gl_resources (texture);
   texture_fbo_free_resources (texture);
 
   clutter_texture_async_load_cancel (texture);
 
-  G_OBJECT_CLASS (clutter_texture_parent_class)->dispose (object);
-}
-
-static void
-clutter_texture_finalize (GObject *object)
-{
-  ClutterTexture *texture = CLUTTER_TEXTURE (object);
-  ClutterTexturePrivate *priv = texture->priv;
-
   if (priv->material != COGL_INVALID_HANDLE)
     {
       cogl_handle_unref (priv->material);
@@ -759,6 +769,14 @@ clutter_texture_finalize (GObject *object)
       priv->pick_material = COGL_INVALID_HANDLE;
     }
 
+  G_OBJECT_CLASS (clutter_texture_parent_class)->dispose (object);
+}
+
+static void
+clutter_texture_finalize (GObject *object)
+{
+  ClutterTexturePrivate *priv = CLUTTER_TEXTURE (object)->priv;
+
   g_free (priv->filename);
 
   G_OBJECT_CLASS (clutter_texture_parent_class)->finalize (object);
@@ -1234,13 +1252,27 @@ clutter_texture_init (ClutterTexture *self)
   priv->repeat_x          = FALSE;
   priv->repeat_y          = FALSE;
   priv->sync_actor_size   = TRUE;
-  priv->material          = cogl_material_new ();
   priv->fbo_handle        = COGL_INVALID_HANDLE;
   priv->pick_material     = COGL_INVALID_HANDLE;
   priv->keep_aspect_ratio = FALSE;
   priv->pick_with_alpha   = FALSE;
   priv->pick_with_alpha_supported = TRUE;
   priv->seen_create_pick_material_warning = FALSE;
+
+  if (G_UNLIKELY (texture_template_material == NULL))
+    {
+      CoglHandle dummy_tex;
+
+      dummy_tex = cogl_texture_new_with_size (1, 1, COGL_TEXTURE_NO_SLICING,
+                                              COGL_PIXEL_FORMAT_RGBA_8888_PRE);
+
+      texture_template_material = cogl_material_new ();
+      cogl_material_set_layer (texture_template_material, 0, dummy_tex);
+      cogl_handle_unref (dummy_tex);
+    }
+
+  g_assert (texture_template_material != NULL);
+  priv->material = cogl_material_copy (texture_template_material);
 }
 
 /**
@@ -1248,12 +1280,13 @@ clutter_texture_init (ClutterTexture *self)
  * @texture: A #ClutterTexture
  *
  * Returns a handle to the underlying COGL material used for drawing
- * the actor. No extra reference is taken so if you need to keep the
- * handle then you should call cogl_handle_ref() on it.
+ * the actor.
+ *
+ * Return value: (transfer none): a handle for a #CoglMaterial. The
+ *   material is owned by the #ClutterTexture and it should not be
+ *   unreferenced
  *
  * Since: 1.0
- *
- * Return value: COGL material handle
  */
 CoglHandle
 clutter_texture_get_cogl_material (ClutterTexture *texture)
@@ -1290,7 +1323,6 @@ clutter_texture_set_cogl_material (ClutterTexture *texture,
 
   cogl_handle_ref (cogl_material);
 
-  /* This */
   if (texture->priv->material)
     cogl_handle_unref (texture->priv->material);
 
@@ -1305,7 +1337,7 @@ clutter_texture_set_cogl_material (ClutterTexture *texture,
 }
 
 /**
- * clutter_texture_get_cogl_texture
+ * clutter_texture_get_cogl_texture:
  * @texture: A #ClutterTexture
  *
  * Retrieves the handle to the underlying COGL texture used for drawing
@@ -1317,23 +1349,35 @@ clutter_texture_set_cogl_material (ClutterTexture *texture,
  * layers you should use clutter_texture_get_cogl_material() instead
  * and use the #CoglMaterial API.
  *
- * Since: 0.8
+ * Return value: (transfer none): a #CoglHandle for the texture. The returned
+ *   handle is owned by the #ClutterTexture and it should not be unreferenced
  *
- * Return value: COGL texture handle
+ * Since: 0.8
  */
 CoglHandle
 clutter_texture_get_cogl_texture (ClutterTexture *texture)
 {
+  ClutterTexturePrivate *priv;
+  CoglMaterialLayerType layer_type;
   const GList *layers;
   int n_layers;
 
   g_return_val_if_fail (CLUTTER_IS_TEXTURE (texture), COGL_INVALID_HANDLE);
 
-  layers = cogl_material_get_layers (texture->priv->material);
+  priv = texture->priv;
+
+  layers = cogl_material_get_layers (priv->material);
+  if (layers == NULL)
+    return COGL_INVALID_HANDLE;
+
   n_layers = g_list_length ((GList *)layers);
   if (n_layers == 0)
     return COGL_INVALID_HANDLE;
 
+  layer_type = cogl_material_layer_get_type (layers->data);
+  if (layer_type != COGL_MATERIAL_LAYER_TYPE_TEXTURE)
+    return COGL_INVALID_HANDLE;
+
   return cogl_material_layer_get_texture (layers->data);
 }
 
@@ -1381,6 +1425,10 @@ clutter_texture_set_cogl_texture (ClutterTexture  *texture,
   texture_free_gl_resources (texture);
 
   /* Use the new texture */
+  if (priv->material == NULL)
+    priv->material = cogl_material_copy (texture_template_material);
+
+  g_assert (priv->material != NULL);
   cogl_material_set_layer (priv->material, 0, cogl_tex);
 
   /* The material now holds a reference to the texture so we can
@@ -1416,7 +1464,7 @@ clutter_texture_set_cogl_texture (ClutterTexture  *texture,
             {
               ClutterRequestMode request;
 
-              if (priv->image_width > priv->image_height)
+              if (priv->image_width >= priv->image_height)
                 request = CLUTTER_REQUEST_HEIGHT_FOR_WIDTH;
               else
                 request = CLUTTER_REQUEST_WIDTH_FOR_HEIGHT;
@@ -2910,6 +2958,11 @@ clutter_texture_set_pick_with_alpha (ClutterTexture *texture,
 
   /* NB: the pick material is created lazily when we first pick */
   priv->pick_with_alpha = pick_with_alpha;
+
+  /* NB: actors are expected to call clutter_actor_queue_redraw when
+   * ever some state changes that will affect painting *or picking...
+   */
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
 }
 
 /**
