@@ -18,18 +18,23 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 /**
  * SECTION:clutter-util
- * @short_description: Misc utility functions.
+ * @short_description: Utility functions
  *
- * Various misc utilility functions.
+ * Various miscellaneous utilility functions.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <glib/gi18n-lib.h>
 
 #include "clutter-util.h"
 #include "clutter-main.h"
@@ -38,17 +43,33 @@
  * clutter_util_next_p2:
  * @a: Value to get the next power
  *
- * Calculates the next power greater than @a.
+ * Calculates the nearest power of two, greater than or equal to @a.
  *
- * Return value: The next power after @a.
+ * Return value: The nearest power of two, greater or equal to @a.
+ *
+ * Deprecated: 1.2
  */
-int 
-clutter_util_next_p2 (int a)
+gint
+clutter_util_next_p2 (gint a)
 {
-  int rval=1;
+  int rval = 1;
 
-  while(rval < a) 
+  while (rval < a)
     rval <<= 1;
 
   return rval;
 }
+
+/*< private >
+ * _clutter_gettext:
+ * @str: a string to localize
+ *
+ * Retrieves the localized version of @str, using the Clutter domain
+ *
+ * Return value: the translated string
+ */
+G_CONST_RETURN gchar *
+_clutter_gettext (const gchar *str)
+{
+  return g_dgettext (GETTEXT_PACKAGE, str);
+}
