@@ -50,6 +50,10 @@ static const GDebugKey cogl_log_debug_keys[] = {
   { "draw", COGL_DEBUG_DRAW },
   { "opengl", COGL_DEBUG_OPENGL },
   { "pango", COGL_DEBUG_PANGO },
+  { "show-source", COGL_DEBUG_SHOW_SOURCE},
+  { "offscreen", COGL_DEBUG_OFFSCREEN },
+  { "texture-pixmap", COGL_DEBUG_TEXTURE_PIXMAP },
+  { "bitmap", COGL_DEBUG_BITMAP }
 };
 static const int n_cogl_log_debug_keys =
   G_N_ELEMENTS (cogl_log_debug_keys);
@@ -58,10 +62,14 @@ static const GDebugKey cogl_behavioural_debug_keys[] = {
   { "rectangles", COGL_DEBUG_RECTANGLES },
   { "disable-batching", COGL_DEBUG_DISABLE_BATCHING },
   { "disable-vbos", COGL_DEBUG_DISABLE_VBOS },
+  { "disable-pbos", COGL_DEBUG_DISABLE_PBOS },
   { "disable-software-transform", COGL_DEBUG_DISABLE_SOFTWARE_TRANSFORM },
-  { "force-scanline-paths", COGL_DEBUG_FORCE_SCANLINE_PATHS },
   { "dump-atlas-image", COGL_DEBUG_DUMP_ATLAS_IMAGE },
-  { "disable-atlas", COGL_DEBUG_DISABLE_ATLAS }
+  { "disable-atlas", COGL_DEBUG_DISABLE_ATLAS },
+  { "disable-texturing", COGL_DEBUG_DISABLE_TEXTURING},
+  { "disable-arbfp", COGL_DEBUG_DISABLE_ARBFP},
+  { "disable-glsl", COGL_DEBUG_DISABLE_GLSL},
+  { "disable-blending", COGL_DEBUG_DISABLE_BLENDING}
 };
 static const int n_cogl_behavioural_debug_keys =
   G_N_ELEMENTS (cogl_behavioural_debug_keys);
@@ -95,30 +103,14 @@ _cogl_parse_debug_string (const char *value,
   else if (strcmp (value, "help") == 0)
     {
       g_printerr ("\n\n%28s\n", "Supported debug values:");
-#define OPT(NAME, HELP) \
-      g_printerr ("%28s %s\n", NAME, HELP);
-      OPT ("handle:", "debug ref counting issues for Cogl objects");
-      OPT ("slicing:", "debug the creation of texture slices");
-      OPT ("atlas:", "debug texture atlas management");
-      OPT ("blend-strings:", "debug blend-string parsing");
-      OPT ("journal:", "view all geometry passing through the journal");
-      OPT ("batching:", "show how geometry is being batched in the journal");
-      OPT ("matrices:", "trace all matrix manipulation");
-      /* XXX: we should replace the "draw" option its very hand wavy... */
-      OPT ("draw:", "misc tracing of some drawing operations");
-      OPT ("pango:", "trace the pango renderer");
-      OPT ("rectangles:", "add wire outlines for all rectangular geometry");
-      OPT ("disable-batching:", "disable the journal batching");
-      OPT ("disable-vbos:", "disable use of OpenGL vertex buffer objects");
-      OPT ("disable-software-transform",
-           "use the GPU to transform rectangular geometry");
-      OPT ("force-scanline-paths:", "use a scanline based path rasterizer");
-      OPT ("dump-atlas-image:", "dump atlas changes to an image file");
-      OPT ("disable-atlas:", "disable texture atlasing");
-      OPT ("opengl:", "traces some select OpenGL calls");
+#define OPT(MASK_NAME, GROUP, NAME, NAME_FORMATTED, DESCRIPTION) \
+      g_printerr ("%28s %s\n", NAME ":", DESCRIPTION);
+#include "cogl-debug-options.h"
       g_printerr ("\n%28s\n", "Special debug values:");
-      OPT ("all:", "Enables all non-behavioural debug options");
-      OPT ("verbose:", "Enables all non-behavioural debug options");
+      OPT (IGNORED, "ignored", "all", "ignored", \
+           "Enables all non-behavioural debug options");
+      OPT (IGNORED, "ignored", "verbose", "ignored", \
+           "Enables all non-behavioural debug options");
 #undef OPT
       exit (1);
     }
