@@ -74,17 +74,75 @@
 #define GL_TEXTURE_3D                           0x806F
 #endif
 
-typedef struct _CoglMaterialBackendARBfpPrivate
+typedef struct _UnitState
 {
-  CoglMaterial *authority_cache;
-  unsigned long authority_cache_age;
+  int constant_id; /* The program.local[] index */
+  unsigned int dirty_combine_constant:1;
 
+  unsigned int sampled:1;
+} UnitState;
+
+typedef struct _ArbfpProgramState
+{
+  int ref_count;
+
+  CoglHandle user_program;
   GString *source;
   GLuint gl_program;
-  gboolean *sampled;
+  UnitState *unit_state;
   int next_constant_id;
+
+  /* We need to track the last material that an ARBfp program was used
+   * with so know if we need to update any program.local parameters. */
+  CoglMaterial *last_used_for_material;
+} ArbfpProgramState;
+
+typedef struct _CoglMaterialBackendARBfpPrivate
+{
+  ArbfpProgramState *arbfp_program_state;
 } CoglMaterialBackendARBfpPrivate;
 
+const CoglMaterialBackend _cogl_material_arbfp_backend;
+
+
+static ArbfpProgramState *
+arbfp_program_state_new (int n_layers)
+{
+  ArbfpProgramState *state = g_slice_new0 (ArbfpProgramState);
+  state->ref_count = 1;
+  state->unit_state = g_new0 (UnitState, n_layers);
+  return state;
+}
+
+static ArbfpProgramState *
+arbfp_program_state_ref (ArbfpProgramState *state)
+{
+  state->ref_count++;
+  return state;
+}
+
+void
+arbfp_program_state_unref (ArbfpProgramState *state)
+{
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  g_return_if_fail (state->ref_count > 0);
+
+  state->ref_count--;
+  if (state->ref_count == 0)
+    {
+      if (state->gl_program)
+        {
+          GE (glDeletePrograms (1, &state->gl_program));
+          state->gl_program = 0;
+        }
+
+      g_free (state->unit_state);
+
+      g_slice_free (ArbfpProgramState, state);
+    }
+}
+
 static int
 _cogl_material_backend_arbfp_get_max_texture_units (void)
 {
@@ -115,9 +173,7 @@ layers_arbfp_would_differ (CoglMaterialLayer **material0_layers,
   /* The layer state that affects arbfp codegen... */
   unsigned long arbfp_codegen_modifiers =
     COGL_MATERIAL_LAYER_STATE_COMBINE |
-    COGL_MATERIAL_LAYER_STATE_COMBINE_CONSTANT |
-    COGL_MATERIAL_LAYER_STATE_UNIT |
-    COGL_MATERIAL_LAYER_STATE_TEXTURE;
+    COGL_MATERIAL_LAYER_STATE_UNIT;
 
   for (i = 0; i < n_layers; i++)
     {
@@ -160,7 +216,7 @@ layers_arbfp_would_differ (CoglMaterialLayer **material0_layers,
  * generate.
  */
 static CoglMaterial *
-find_arbfp_authority (CoglMaterial *material)
+find_arbfp_authority (CoglMaterial *material, CoglHandle user_program)
 {
   CoglMaterial *authority0;
   CoglMaterial *authority1;
@@ -171,6 +227,9 @@ find_arbfp_authority (CoglMaterial *material)
   /* XXX: we'll need to update this when we add fog support to the
    * arbfp codegen */
 
+  if (user_program != COGL_INVALID_HANDLE)
+    return material;
+
   /* Find the first material that modifies state that affects the
    * arbfp codegen... */
   authority0 = _cogl_material_get_authority (material,
@@ -200,17 +259,17 @@ find_arbfp_authority (CoglMaterial *material)
         g_alloca (sizeof (CoglMaterialLayer *) * n_layers);
       state.i = 0;
       state.layers = authority0_layers;
-      _cogl_material_foreach_layer (authority0,
-                                    add_layer_to_array_cb,
-                                    &state);
+      _cogl_material_foreach_layer_internal (authority0,
+                                             add_layer_to_array_cb,
+                                             &state);
 
       authority1_layers =
         g_alloca (sizeof (CoglMaterialLayer *) * n_layers);
       state.i = 0;
       state.layers = authority1_layers;
-      _cogl_material_foreach_layer (authority1,
-                                    add_layer_to_array_cb,
-                                    &state);
+      _cogl_material_foreach_layer_internal (authority1,
+                                             add_layer_to_array_cb,
+                                             &state);
 
       if (layers_arbfp_would_differ (authority0_layers, authority1_layers,
                                      n_layers))
@@ -233,16 +292,34 @@ find_arbfp_authority (CoglMaterial *material)
   return authority1;
 }
 
-static void
-invalidate_arbfp_authority_cache (CoglMaterial *material)
+static CoglMaterialBackendARBfpPrivate *
+get_arbfp_priv (CoglMaterial *material)
 {
-  if (material->backend_priv_set_mask  & COGL_MATERIAL_BACKEND_ARBFP_MASK)
+  if (!(material->backend_priv_set_mask & COGL_MATERIAL_BACKEND_ARBFP_MASK))
+    return NULL;
+
+  return material->backend_privs[COGL_MATERIAL_BACKEND_ARBFP];
+}
+
+static void
+set_arbfp_priv (CoglMaterial *material, CoglMaterialBackendARBfpPrivate *priv)
+{
+  if (priv)
     {
-      CoglMaterialBackendARBfpPrivate *priv =
-        material->backend_privs[COGL_MATERIAL_BACKEND_ARBFP];
-      priv->authority_cache = NULL;
-      priv->authority_cache_age = 0;
+      material->backend_privs[COGL_MATERIAL_BACKEND_ARBFP] = priv;
+      material->backend_priv_set_mask |= COGL_MATERIAL_BACKEND_ARBFP_MASK;
     }
+  else
+    material->backend_priv_set_mask &= ~COGL_MATERIAL_BACKEND_ARBFP_MASK;
+}
+
+static ArbfpProgramState *
+get_arbfp_program_state (CoglMaterial *material)
+{
+  CoglMaterialBackendARBfpPrivate *priv = get_arbfp_priv (material);
+  if (!priv)
+    return NULL;
+  return priv->arbfp_program_state;
 }
 
 static gboolean
@@ -250,90 +327,103 @@ _cogl_material_backend_arbfp_start (CoglMaterial *material,
                                     int n_layers,
                                     unsigned long materials_difference)
 {
-  CoglMaterial *authority;
   CoglMaterialBackendARBfpPrivate *priv;
+  CoglMaterial *authority;
   CoglMaterialBackendARBfpPrivate *authority_priv;
+  CoglHandle user_program;
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
-  if (!_cogl_features_available_private (COGL_FEATURE_PRIVATE_ARB_FP))
+  /* First validate that we can handle the current state using ARBfp
+   */
+
+  if (!cogl_features_available (COGL_FEATURE_SHADERS_ARBFP))
     return FALSE;
 
   /* TODO: support fog */
   if (ctx->legacy_fog_state.enabled)
     return FALSE;
 
-  /* Note: we allocate ARBfp private state for both the given material
-   * and the authority. (The oldest ancestor whos state will result in
-   * the same program being generated) The former will simply cache a
-   * pointer to the authority and the later will track the arbfp
-   * program that we will generate.
+  user_program = cogl_material_get_user_program (material);
+  if (user_program != COGL_INVALID_HANDLE &&
+      _cogl_program_get_language (user_program) != COGL_SHADER_LANGUAGE_ARBFP)
+    return FALSE;
+
+  /* Now lookup our ARBfp backend private state (allocating if
+   * necessary) */
+  priv = get_arbfp_priv (material);
+  if (!priv)
+    {
+      priv = g_slice_new0 (CoglMaterialBackendARBfpPrivate);
+      set_arbfp_priv (material, priv);
+    }
+
+  /* If we have a valid arbfp_program_state pointer then we are all
+   * set and don't need to generate a new program. */
+  if (priv->arbfp_program_state)
+    return TRUE;
+
+  /* If we don't have an associated arbfp program yet then find the
+   * arbfp-authority (the oldest ancestor whose state will result in
+   * the same program being generated as for this material).
+   *
+   * We always make sure to associate new programs with the
+   * arbfp-authority to maximize the chance that other materials can
+   * share it.
    */
-
-  if (!(material->backend_priv_set_mask & COGL_MATERIAL_BACKEND_ARBFP_MASK))
+  authority = find_arbfp_authority (material, user_program);
+  authority_priv = get_arbfp_priv (authority);
+  if (!authority_priv)
     {
-      material->backend_privs[COGL_MATERIAL_BACKEND_ARBFP] =
-        g_slice_new0 (CoglMaterialBackendARBfpPrivate);
-      material->backend_priv_set_mask |= COGL_MATERIAL_BACKEND_ARBFP_MASK;
+      authority_priv = g_slice_new0 (CoglMaterialBackendARBfpPrivate);
+      set_arbfp_priv (authority, authority_priv);
     }
-  priv = material->backend_privs[COGL_MATERIAL_BACKEND_ARBFP];
 
-  /* XXX: We are making assumptions that we don't yet support
-   * modification of ancestors to optimize the sharing of state in the
-   * material graph. When we start to support this then the arbfp
-   * backend will somehow need to be notified of graph changes that
-   * may invalidate authority_cache pointers.
+  /* If we don't have an existing program associated with the
+   * arbfp-authority then start generating code for a new program...
    */
-
-  if (priv->authority_cache &&
-      priv->authority_cache_age != _cogl_material_get_age (material))
-    invalidate_arbfp_authority_cache (material);
-
-  if (!priv->authority_cache)
+  if (!authority_priv->arbfp_program_state)
     {
-      priv->authority_cache = find_arbfp_authority (material);
-      priv->authority_cache_age = _cogl_material_get_age (material);
+      ArbfpProgramState *arbfp_program_state =
+        arbfp_program_state_new (n_layers);
+      authority_priv->arbfp_program_state = arbfp_program_state;
+
+      arbfp_program_state->user_program = user_program;
+      if (user_program == COGL_INVALID_HANDLE)
+        {
+          int i;
+
+          /* We reuse a single grow-only GString for ARBfp code-gen */
+          g_string_set_size (ctx->arbfp_source_buffer, 0);
+          arbfp_program_state->source = ctx->arbfp_source_buffer;
+          g_string_append (arbfp_program_state->source,
+                           "!!ARBfp1.0\n"
+                           "TEMP output;\n"
+                           "TEMP tmp0, tmp1, tmp2, tmp3, tmp4;\n"
+                           "PARAM half = {.5, .5, .5, .5};\n"
+                           "PARAM one = {1, 1, 1, 1};\n"
+                           "PARAM two = {2, 2, 2, 2};\n"
+                           "PARAM minus_one = {-1, -1, -1, -1};\n");
+
+          for (i = 0; i < n_layers; i++)
+            {
+              arbfp_program_state->unit_state[i].sampled = FALSE;
+              arbfp_program_state->unit_state[i].dirty_combine_constant = FALSE;
+            }
+          arbfp_program_state->next_constant_id = 0;
+        }
     }
 
-  authority = priv->authority_cache;
-  if (!(authority->backend_priv_set_mask & COGL_MATERIAL_BACKEND_ARBFP_MASK))
-    {
-      authority->backend_privs[COGL_MATERIAL_BACKEND_ARBFP] =
-        g_slice_new0 (CoglMaterialBackendARBfpPrivate);
-      authority->backend_priv_set_mask |= COGL_MATERIAL_BACKEND_ARBFP_MASK;
-    }
-  authority_priv = authority->backend_privs[COGL_MATERIAL_BACKEND_ARBFP];
-
-  if (authority_priv->gl_program == 0)
-    {
-      /* We reuse a single grow-only GString for ARBfp code-gen */
-      g_string_set_size (ctx->arbfp_source_buffer, 0);
-      authority_priv->source = ctx->arbfp_source_buffer;
-      g_string_append (authority_priv->source,
-                       "!!ARBfp1.0\n"
-                       "TEMP output;\n"
-                       "TEMP tmp0, tmp1, tmp2, tmp3, tmp4;\n"
-                       "PARAM half = {.5, .5, .5, .5};\n"
-                       "PARAM one = {1, 1, 1, 1};\n"
-                       "PARAM two = {2, 2, 2, 2};\n"
-                       "PARAM minus_one = {-1, -1, -1, -1};\n");
-      authority_priv->sampled = g_new0 (gboolean, n_layers);
-    }
+  /* Finally, if the material isn't actually its own arbfp-authority
+   * then steal a reference to the program state associated with the
+   * arbfp-authority... */
+  if (authority != material)
+    priv->arbfp_program_state =
+      arbfp_program_state_ref (authority_priv->arbfp_program_state);
 
   return TRUE;
 }
 
-static CoglMaterial *
-get_arbfp_authority (CoglMaterial *material)
-{
-  CoglMaterialBackendARBfpPrivate *priv =
-    material->backend_privs[COGL_MATERIAL_BACKEND_ARBFP];
-
-  g_return_val_if_fail (priv != NULL, NULL);
-
-  return priv->authority_cache;
-}
-
 /* Determines if we need to handle the RGB and A texture combining
  * separately or is the same function used for both channel masks and
  * with the same arguments...
@@ -428,13 +518,13 @@ gl_target_to_arbfp_string (GLenum gl_target)
 }
 
 static void
-setup_texture_source (CoglMaterialBackendARBfpPrivate *priv,
+setup_texture_source (ArbfpProgramState *arbfp_program_state,
                       int unit_index,
                       GLenum gl_target)
 {
-  if (!priv->sampled[unit_index])
+  if (!arbfp_program_state->unit_state[unit_index].sampled)
     {
-      g_string_append_printf (priv->source,
+      g_string_append_printf (arbfp_program_state->source,
                               "TEMP texel%d;\n"
                               "TEX texel%d,fragment.texcoord[%d],"
                               "texture[%d],%s;\n",
@@ -443,7 +533,7 @@ setup_texture_source (CoglMaterialBackendARBfpPrivate *priv,
                               unit_index,
                               unit_index,
                               gl_target_to_arbfp_string (gl_target));
-      priv->sampled[unit_index] = TRUE;
+      arbfp_program_state->unit_state[unit_index].sampled = TRUE;
     }
 }
 
@@ -481,7 +571,7 @@ append_arg (GString *source, const CoglMaterialBackendARBfpArg *arg)
                               arg->texture_unit, arg->swizzle);
       break;
     case COGL_MATERIAL_BACKEND_ARBFP_ARG_TYPE_CONSTANT:
-      g_string_append_printf (source, "constant%d%s",
+      g_string_append_printf (source, "program.local[%d]%s",
                               arg->constant_id, arg->swizzle);
       break;
     case COGL_MATERIAL_BACKEND_ARBFP_ARG_TYPE_SIMPLE:
@@ -503,9 +593,7 @@ setup_arg (CoglMaterial *material,
            GLint op,
            CoglMaterialBackendARBfpArg *arg)
 {
-  CoglMaterial *arbfp_authority = get_arbfp_authority (material);
-  CoglMaterialBackendARBfpPrivate *priv =
-    arbfp_authority->backend_privs[COGL_MATERIAL_BACKEND_ARBFP];
+  ArbfpProgramState *arbfp_program_state = get_arbfp_program_state (material);
   static const char *tmp_name[3] = { "tmp0", "tmp1", "tmp2" };
   GLenum gl_target;
   CoglHandle texture;
@@ -518,33 +606,19 @@ setup_arg (CoglMaterial *material,
       arg->texture_unit = _cogl_material_layer_get_unit_index (layer);
       texture = _cogl_material_layer_get_texture (layer);
       cogl_texture_get_gl_texture (texture, NULL, &gl_target);
-      setup_texture_source (priv, arg->texture_unit, gl_target);
+      setup_texture_source (arbfp_program_state, arg->texture_unit, gl_target);
       break;
     case GL_CONSTANT:
       {
-        unsigned long state = COGL_MATERIAL_LAYER_STATE_COMBINE_CONSTANT;
-        CoglMaterialLayer *authority =
-          _cogl_material_layer_get_authority (layer, state);
-        CoglMaterialLayerBigState *big_state = authority->big_state;
-        char buf[4][G_ASCII_DTOSTR_BUF_SIZE];
-        int i;
+        int unit_index = _cogl_material_layer_get_unit_index (layer);
+        UnitState *unit_state = &arbfp_program_state->unit_state[unit_index];
+
+        unit_state->constant_id = arbfp_program_state->next_constant_id++;
+        unit_state->dirty_combine_constant = TRUE;
 
         arg->type = COGL_MATERIAL_BACKEND_ARBFP_ARG_TYPE_CONSTANT;
-        arg->name = "constant%d";
-        arg->constant_id = priv->next_constant_id++;
-
-        for (i = 0; i < 4; i++)
-          g_ascii_dtostr (buf[i], G_ASCII_DTOSTR_BUF_SIZE,
-                          big_state->texture_combine_constant[i]);
-
-        g_string_append_printf (priv->source,
-                                "PARAM constant%d = "
-                                "  {%s, %s, %s, %s};\n",
-                                arg->constant_id,
-                                buf[0],
-                                buf[1],
-                                buf[2],
-                                buf[3]);
+        arg->name = "program.local[%d]";
+        arg->constant_id = unit_state->constant_id;
         break;
       }
     case GL_PRIMARY_COLOR:
@@ -564,7 +638,7 @@ setup_arg (CoglMaterial *material,
       arg->texture_unit = src - GL_TEXTURE0;
       texture = _cogl_material_layer_get_texture (layer);
       cogl_texture_get_gl_texture (texture, NULL, &gl_target);
-      setup_texture_source (priv, arg->texture_unit, gl_target);
+      setup_texture_source (arbfp_program_state, arg->texture_unit, gl_target);
     }
 
   arg->swizzle = "";
@@ -574,11 +648,11 @@ setup_arg (CoglMaterial *material,
     case GL_SRC_COLOR:
       break;
     case GL_ONE_MINUS_SRC_COLOR:
-      g_string_append_printf (priv->source,
+      g_string_append_printf (arbfp_program_state->source,
                               "SUB tmp%d, one, ",
                               arg_index);
-      append_arg (priv->source, arg);
-      g_string_append_printf (priv->source, ";\n");
+      append_arg (arbfp_program_state->source, arg);
+      g_string_append_printf (arbfp_program_state->source, ";\n");
       arg->type = COGL_MATERIAL_BACKEND_ARBFP_ARG_TYPE_SIMPLE;
       arg->name = tmp_name[arg_index];
       arg->swizzle = "";
@@ -590,16 +664,16 @@ setup_arg (CoglMaterial *material,
         arg->swizzle = ".a";
       break;
     case GL_ONE_MINUS_SRC_ALPHA:
-      g_string_append_printf (priv->source,
+      g_string_append_printf (arbfp_program_state->source,
                               "SUB tmp%d, one, ",
                               arg_index);
-      append_arg (priv->source, arg);
+      append_arg (arbfp_program_state->source, arg);
       /* avoid a swizzle if we know RGB are going to be masked
        * in the end anyway */
       if (mask != COGL_BLEND_STRING_CHANNEL_MASK_ALPHA)
-        g_string_append_printf (priv->source, ".a;\n");
+        g_string_append_printf (arbfp_program_state->source, ".a;\n");
       else
-        g_string_append_printf (priv->source, ";\n");
+        g_string_append_printf (arbfp_program_state->source, ";\n");
       arg->type = COGL_MATERIAL_BACKEND_ARBFP_ARG_TYPE_SIMPLE;
       arg->name = tmp_name[arg_index];
       break;
@@ -644,9 +718,7 @@ append_function (CoglMaterial *material,
                  CoglMaterialBackendARBfpArg *args,
                  int n_args)
 {
-  CoglMaterial *arbfp_authority = get_arbfp_authority (material);
-  CoglMaterialBackendARBfpPrivate *priv =
-    arbfp_authority->backend_privs[COGL_MATERIAL_BACKEND_ARBFP];
+  ArbfpProgramState *arbfp_program_state = get_arbfp_program_state (material);
   const char *mask_name;
 
   switch (mask)
@@ -668,33 +740,36 @@ append_function (CoglMaterial *material,
   switch (function)
     {
     case GL_ADD:
-      g_string_append_printf (priv->source, "ADD_SAT output%s, ",
+      g_string_append_printf (arbfp_program_state->source,
+                              "ADD_SAT output%s, ",
                               mask_name);
       break;
     case GL_MODULATE:
       /* Note: no need to saturate since we can assume operands
        * have values in the range [0,1] */
-      g_string_append_printf (priv->source, "MUL output%s, ",
+      g_string_append_printf (arbfp_program_state->source, "MUL output%s, ",
                               mask_name);
       break;
     case GL_REPLACE:
       /* Note: no need to saturate since we can assume operand
        * has a value in the range [0,1] */
-      g_string_append_printf (priv->source, "MOV output%s, ",
+      g_string_append_printf (arbfp_program_state->source, "MOV output%s, ",
                               mask_name);
       break;
     case GL_SUBTRACT:
-      g_string_append_printf (priv->source, "SUB_SAT output%s, ",
+      g_string_append_printf (arbfp_program_state->source,
+                              "SUB_SAT output%s, ",
                               mask_name);
       break;
     case GL_ADD_SIGNED:
-      g_string_append_printf (priv->source, "ADD tmp3%s, ",
+      g_string_append_printf (arbfp_program_state->source, "ADD tmp3%s, ",
                               mask_name);
-      append_arg (priv->source, &args[0]);
-      g_string_append (priv->source, ", ");
-      append_arg (priv->source, &args[1]);
-      g_string_append (priv->source, ";\n");
-      g_string_append_printf (priv->source, "SUB_SAT output%s, tmp3, half",
+      append_arg (arbfp_program_state->source, &args[0]);
+      g_string_append (arbfp_program_state->source, ", ");
+      append_arg (arbfp_program_state->source, &args[1]);
+      g_string_append (arbfp_program_state->source, ";\n");
+      g_string_append_printf (arbfp_program_state->source,
+                              "SUB_SAT output%s, tmp3, half",
                               mask_name);
       n_args = 0;
       break;
@@ -722,20 +797,20 @@ append_function (CoglMaterial *material,
          * output = 4 * DP3 (src0 - 0.5, src1 - 0.5)
          */
 
-        g_string_append (priv->source, "MAD tmp3, two, ");
-        append_arg (priv->source, &args[0]);
-        g_string_append (priv->source, ", minus_one;\n");
+        g_string_append (arbfp_program_state->source, "MAD tmp3, two, ");
+        append_arg (arbfp_program_state->source, &args[0]);
+        g_string_append (arbfp_program_state->source, ", minus_one;\n");
 
         if (!backend_arbfp_args_equal (&args[0], &args[1]))
           {
-            g_string_append (priv->source, "MAD tmp4, two, ");
-            append_arg (priv->source, &args[1]);
-            g_string_append (priv->source, ", minus_one;\n");
+            g_string_append (arbfp_program_state->source, "MAD tmp4, two, ");
+            append_arg (arbfp_program_state->source, &args[1]);
+            g_string_append (arbfp_program_state->source, ", minus_one;\n");
           }
         else
           tmp4 = "tmp3";
 
-        g_string_append_printf (priv->source,
+        g_string_append_printf (arbfp_program_state->source,
                                 "DP3_SAT output%s, tmp3, %s",
                                 mask_name, tmp4);
         n_args = 0;
@@ -747,31 +822,31 @@ append_function (CoglMaterial *material,
 
       /* NB: GL_INTERPOLATE = arg0*arg2 + arg1*(1-arg2)
        * but LRP dst, a, b, c = b*a + c*(1-a) */
-      g_string_append_printf (priv->source, "LRP output%s, ",
+      g_string_append_printf (arbfp_program_state->source, "LRP output%s, ",
                               mask_name);
-      append_arg (priv->source, &args[2]);
-      g_string_append (priv->source, ", ");
-      append_arg (priv->source, &args[0]);
-      g_string_append (priv->source, ", ");
-      append_arg (priv->source, &args[1]);
+      append_arg (arbfp_program_state->source, &args[2]);
+      g_string_append (arbfp_program_state->source, ", ");
+      append_arg (arbfp_program_state->source, &args[0]);
+      g_string_append (arbfp_program_state->source, ", ");
+      append_arg (arbfp_program_state->source, &args[1]);
       n_args = 0;
       break;
     default:
       g_error ("Unknown texture combine function %d", function);
-      g_string_append_printf (priv->source, "MUL_SAT output%s, ",
+      g_string_append_printf (arbfp_program_state->source, "MUL_SAT output%s, ",
                               mask_name);
       n_args = 2;
       break;
     }
 
   if (n_args > 0)
-    append_arg (priv->source, &args[0]);
+    append_arg (arbfp_program_state->source, &args[0]);
   if (n_args > 1)
     {
-      g_string_append (priv->source, ", ");
-      append_arg (priv->source, &args[1]);
+      g_string_append (arbfp_program_state->source, ", ");
+      append_arg (arbfp_program_state->source, &args[1]);
     }
-  g_string_append (priv->source, ";\n");
+  g_string_append (arbfp_program_state->source, ";\n");
 }
 
 static void
@@ -811,9 +886,7 @@ _cogl_material_backend_arbfp_add_layer (CoglMaterial *material,
                                         CoglMaterialLayer *layer,
                                         unsigned long layers_difference)
 {
-  CoglMaterial *arbfp_authority = get_arbfp_authority (material);
-  CoglMaterialBackendARBfpPrivate *priv =
-    arbfp_authority->backend_privs[COGL_MATERIAL_BACKEND_ARBFP];
+  ArbfpProgramState *arbfp_program_state = get_arbfp_program_state (material);
   CoglMaterialLayer *combine_authority =
     _cogl_material_layer_get_authority (layer,
                                         COGL_MATERIAL_LAYER_STATE_COMBINE);
@@ -848,7 +921,7 @@ _cogl_material_backend_arbfp_add_layer (CoglMaterial *material,
    * We are careful to only saturate when writing to output.
    */
 
-  if (!priv->source)
+  if (!arbfp_program_state->source)
     return TRUE;
 
   if (!need_texture_combine_separate (combine_authority))
@@ -894,14 +967,45 @@ _cogl_material_backend_arbfp_add_layer (CoglMaterial *material,
 gboolean
 _cogl_material_backend_arbfp_passthrough (CoglMaterial *material)
 {
-  CoglMaterial *arbfp_authority = get_arbfp_authority (material);
-  CoglMaterialBackendARBfpPrivate *priv =
-    arbfp_authority->backend_privs[COGL_MATERIAL_BACKEND_ARBFP];
+  ArbfpProgramState *arbfp_program_state = get_arbfp_program_state (material);
 
-  if (!priv->source)
+  if (!arbfp_program_state->source)
     return TRUE;
 
-  g_string_append (priv->source, "MOV output, fragment.color.primary;\n");
+  g_string_append (arbfp_program_state->source,
+                   "MOV output, fragment.color.primary;\n");
+  return TRUE;
+}
+
+typedef struct _UpdateConstantsState
+{
+  int unit;
+  gboolean update_all;
+  ArbfpProgramState *arbfp_program_state;
+} UpdateConstantsState;
+
+static gboolean
+update_constants_cb (CoglMaterial *material,
+                     int layer_index,
+                     void *user_data)
+{
+  UpdateConstantsState *state = user_data;
+  ArbfpProgramState *arbfp_program_state = state->arbfp_program_state;
+  UnitState *unit_state = &arbfp_program_state->unit_state[state->unit++];
+
+  _COGL_GET_CONTEXT (ctx, FALSE);
+
+  if (state->update_all || unit_state->dirty_combine_constant)
+    {
+      float constant[4];
+      _cogl_material_get_layer_combine_constant (material,
+                                                 layer_index,
+                                                 constant);
+      GE (glProgramLocalParameter4fv (GL_FRAGMENT_PROGRAM_ARB,
+                                      unit_state->constant_id,
+                                      constant));
+      unit_state->dirty_combine_constant = FALSE;
+    }
   return TRUE;
 }
 
@@ -909,13 +1013,12 @@ static gboolean
 _cogl_material_backend_arbfp_end (CoglMaterial *material,
                                   unsigned long materials_difference)
 {
-  CoglMaterial *arbfp_authority = get_arbfp_authority (material);
-  CoglMaterialBackendARBfpPrivate *priv =
-    arbfp_authority->backend_privs[COGL_MATERIAL_BACKEND_ARBFP];
+  ArbfpProgramState *arbfp_program_state = get_arbfp_program_state (material);
+  GLuint gl_program;
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
-  if (priv->source)
+  if (arbfp_program_state->source)
     {
       GLenum gl_error;
       COGL_STATIC_COUNTER (backend_arbfp_compile_counter,
@@ -926,92 +1029,139 @@ _cogl_material_backend_arbfp_end (CoglMaterial *material,
 
       COGL_COUNTER_INC (_cogl_uprof_context, backend_arbfp_compile_counter);
 
-      g_string_append (priv->source, "MOV result.color,output;\n");
-      g_string_append (priv->source, "END\n");
+      g_string_append (arbfp_program_state->source,
+                       "MOV result.color,output;\n");
+      g_string_append (arbfp_program_state->source, "END\n");
 
       if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_SHOW_SOURCE))
-        g_message ("material program:\n%s", priv->source->str);
+        g_message ("material program:\n%s", arbfp_program_state->source->str);
 
-      GE (glGenPrograms (1, &priv->gl_program));
+      GE (glGenPrograms (1, &arbfp_program_state->gl_program));
 
-      GE (glBindProgram (GL_FRAGMENT_PROGRAM_ARB, priv->gl_program));
+      GE (glBindProgram (GL_FRAGMENT_PROGRAM_ARB,
+                         arbfp_program_state->gl_program));
 
       while ((gl_error = glGetError ()) != GL_NO_ERROR)
         ;
       glProgramString (GL_FRAGMENT_PROGRAM_ARB,
                        GL_PROGRAM_FORMAT_ASCII_ARB,
-                       priv->source->len,
-                       priv->source->str);
+                       arbfp_program_state->source->len,
+                       arbfp_program_state->source->str);
       if (glGetError () != GL_NO_ERROR)
         {
           g_warning ("\n%s\n%s",
-                     priv->source->str,
+                     arbfp_program_state->source->str,
                      glGetString (GL_PROGRAM_ERROR_STRING_ARB));
         }
 
-      priv->source = NULL;
+      arbfp_program_state->source = NULL;
+    }
 
-      g_free (priv->sampled);
-      priv->sampled = NULL;
+  if (arbfp_program_state->user_program != COGL_INVALID_HANDLE)
+    {
+      CoglProgram *program = (CoglProgram *)arbfp_program_state->user_program;
+      gl_program = program->gl_handle;
     }
   else
-    GE (glBindProgram (GL_FRAGMENT_PROGRAM_ARB, priv->gl_program));
+    gl_program = arbfp_program_state->gl_program;
 
+  GE (glBindProgram (GL_FRAGMENT_PROGRAM_ARB, gl_program));
   _cogl_use_program (COGL_INVALID_HANDLE, COGL_MATERIAL_PROGRAM_TYPE_ARBFP);
 
+  if (arbfp_program_state->user_program == COGL_INVALID_HANDLE)
+    {
+      UpdateConstantsState state;
+      state.unit = 0;
+      state.arbfp_program_state = arbfp_program_state;
+      /* If this arbfp program was last used with a different material
+       * then we need to ensure we update all program.local params */
+      state.update_all =
+        material != arbfp_program_state->last_used_for_material;
+      cogl_material_foreach_layer (material,
+                                   update_constants_cb,
+                                   &state);
+    }
+
+  /* We need to track what material used this arbfp program last since
+   * we will need to update program.local params when switching
+   * between different materials. */
+  arbfp_program_state->last_used_for_material = material;
+
   return TRUE;
 }
 
+static void
+dirty_arbfp_program_state (CoglMaterial *material)
+{
+  CoglMaterialBackendARBfpPrivate *priv;
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  priv = get_arbfp_priv (material);
+  if (!priv)
+    return;
+
+  if (priv->arbfp_program_state)
+    {
+      arbfp_program_state_unref (priv->arbfp_program_state);
+      priv->arbfp_program_state = NULL;
+    }
+}
+
 static void
 _cogl_material_backend_arbfp_material_pre_change_notify (
                                                    CoglMaterial *material,
                                                    CoglMaterialState change,
                                                    const CoglColor *new_color)
 {
-  CoglMaterialBackendARBfpPrivate *priv =
-    material->backend_privs[COGL_MATERIAL_BACKEND_ARBFP];
   static const unsigned long fragment_op_changes =
-    COGL_MATERIAL_STATE_LAYERS;
+    COGL_MATERIAL_STATE_LAYERS |
+    COGL_MATERIAL_STATE_USER_SHADER;
     /* TODO: COGL_MATERIAL_STATE_FOG */
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+  if (!(change & fragment_op_changes))
+    return;
 
-  if (material->backend_priv_set_mask & COGL_MATERIAL_BACKEND_ARBFP_MASK &&
-      priv->gl_program &&
-      change & fragment_op_changes)
-    {
-      GE (glDeletePrograms (1, &priv->gl_program));
-      priv->gl_program = 0;
-    }
-}
-
-static gboolean
-invalidate_arbfp_authority_cache_cb (CoglMaterialNode *node,
-                                     void *user_data)
-{
-  CoglMaterial *material = COGL_MATERIAL (node);
-  invalidate_arbfp_authority_cache (material);
-  return TRUE;
-}
-
-static void
-_cogl_material_backend_arbfp_material_set_parent_notify (
-                                                CoglMaterial *material)
-{
-  /* Any arbfp authority cache associated with this material or
-   * any of its descendants will now be invalid. */
-  invalidate_arbfp_authority_cache (material);
-
-  _cogl_material_node_foreach_child (COGL_MATERIAL_NODE (material),
-                                     invalidate_arbfp_authority_cache_cb,
-                                     NULL);
+  dirty_arbfp_program_state (material);
 }
 
+/* NB: layers are considered immutable once they have any dependants
+ * so although multiple materials can end up depending on a single
+ * static layer, we can guarantee that if a layer is being *changed*
+ * then it can only have one material depending on it.
+ *
+ * XXX: Don't forget this is *pre* change, we can't read the new value
+ * yet!
+ */
 static void
 _cogl_material_backend_arbfp_layer_pre_change_notify (
+                                                CoglMaterial *owner,
                                                 CoglMaterialLayer *layer,
-                                                CoglMaterialLayerState changes)
+                                                CoglMaterialLayerState change)
 {
+  CoglMaterialBackendARBfpPrivate *priv;
+  static const unsigned long not_fragment_op_changes =
+    COGL_MATERIAL_LAYER_STATE_COMBINE_CONSTANT |
+    COGL_MATERIAL_LAYER_STATE_TEXTURE;
+
+  priv = get_arbfp_priv (owner);
+  if (!priv)
+    return;
+
+  if (!(change & not_fragment_op_changes))
+    {
+      dirty_arbfp_program_state (owner);
+      return;
+    }
+
+  if (change & COGL_MATERIAL_LAYER_STATE_COMBINE_CONSTANT)
+    {
+      ArbfpProgramState *arbfp_program_state =
+        get_arbfp_program_state (owner);
+      int unit_index = _cogl_material_layer_get_unit_index (layer);
+      arbfp_program_state->unit_state[unit_index].dirty_combine_constant = TRUE;
+    }
+
   /* TODO: we could be saving snippets of texture combine code along
    * with each layer and then when a layer changes we would just free
    * the snippet. */
@@ -1021,18 +1171,13 @@ _cogl_material_backend_arbfp_layer_pre_change_notify (
 static void
 _cogl_material_backend_arbfp_free_priv (CoglMaterial *material)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  if (material->backend_priv_set_mask & COGL_MATERIAL_BACKEND_ARBFP_MASK)
+  CoglMaterialBackendARBfpPrivate *priv = get_arbfp_priv (material);
+  if (priv)
     {
-      CoglMaterialBackendARBfpPrivate *priv =
-        material->backend_privs[COGL_MATERIAL_BACKEND_ARBFP];
-
-      glDeletePrograms (1, &priv->gl_program);
-      if (priv->sampled)
-        g_free (priv->sampled);
+      if (priv->arbfp_program_state)
+        arbfp_program_state_unref (priv->arbfp_program_state);
       g_slice_free (CoglMaterialBackendARBfpPrivate, priv);
-      material->backend_priv_set_mask &= ~COGL_MATERIAL_BACKEND_ARBFP_MASK;
+      set_arbfp_priv (material, NULL);
     }
 }
 
@@ -1044,7 +1189,7 @@ const CoglMaterialBackend _cogl_material_arbfp_backend =
   _cogl_material_backend_arbfp_passthrough,
   _cogl_material_backend_arbfp_end,
   _cogl_material_backend_arbfp_material_pre_change_notify,
-  _cogl_material_backend_arbfp_material_set_parent_notify,
+  NULL,
   _cogl_material_backend_arbfp_layer_pre_change_notify,
   _cogl_material_backend_arbfp_free_priv,
   NULL
