@@ -37,13 +37,14 @@
 #include "cogl-context.h"
 #include "cogl-handle.h"
 #include "cogl-journal-private.h"
+#include "cogl-material-opengl-private.h"
 
 #include <string.h>
 #include <math.h>
 
 static void _cogl_texture_2d_free (CoglTexture2D *tex_2d);
 
-COGL_HANDLE_DEFINE (Texture2D, texture_2d);
+COGL_TEXTURE_INTERNAL_DEFINE (Texture2D, texture_2d);
 
 static const CoglTextureVtable cogl_texture_2d_vtable;
 
@@ -127,7 +128,7 @@ static void
 _cogl_texture_2d_set_wrap_mode_parameters (CoglTexture *tex,
                                            GLenum wrap_mode_s,
                                            GLenum wrap_mode_t,
-                                           GLenum wrap_mode_r)
+                                           GLenum wrap_mode_p)
 {
   CoglTexture2D *tex_2d = COGL_TEXTURE_2D (tex);
 
@@ -137,7 +138,9 @@ _cogl_texture_2d_set_wrap_mode_parameters (CoglTexture *tex,
   if (tex_2d->wrap_mode_s != wrap_mode_s ||
       tex_2d->wrap_mode_t != wrap_mode_t)
     {
-      GE( glBindTexture (GL_TEXTURE_2D, tex_2d->gl_texture) );
+      _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
+                                       tex_2d->gl_texture,
+                                       FALSE);
       GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap_mode_s) );
       GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap_mode_t) );
 
@@ -149,15 +152,10 @@ _cogl_texture_2d_set_wrap_mode_parameters (CoglTexture *tex,
 static void
 _cogl_texture_2d_free (CoglTexture2D *tex_2d)
 {
-  GE( glDeleteTextures (1, &tex_2d->gl_texture) );
-  g_free (tex_2d);
-}
+  _cogl_delete_gl_texture (tex_2d->gl_texture);
 
-static gboolean
-_cogl_texture_2d_is_pot (unsigned int num)
-{
-  /* Make sure there is only one bit set */
-  return (num & (num - 1)) == 0;
+  /* Chain up */
+  _cogl_texture_free (COGL_TEXTURE (tex_2d));
 }
 
 static gboolean
@@ -168,18 +166,11 @@ _cogl_texture_2d_can_create (unsigned int width,
   GLenum gl_intformat;
   GLenum gl_type;
 
-  /* If the driver doesn't support glGenerateMipmap then we need to
-     store a copy of the first pixels to cause an update. Instead of
-     duplicating the code here we'll just make it fallback to
-     CoglTexture2DSliced */
-  if (!cogl_features_available (COGL_FEATURE_OFFSCREEN))
-    return FALSE;
-
   /* If NPOT textures aren't supported then the size must be a power
      of two */
   if (!cogl_features_available (COGL_FEATURE_TEXTURE_NPOT) &&
-      (!_cogl_texture_2d_is_pot (width) ||
-       !_cogl_texture_2d_is_pot (height)))
+      (!_cogl_util_is_pot (width) ||
+       !_cogl_util_is_pot (height)))
     return FALSE;
 
   _cogl_pixel_format_to_gl (internal_format,
@@ -253,7 +244,9 @@ _cogl_texture_2d_new_with_size (unsigned int     width,
   tex_2d = _cogl_texture_2d_create_base (width, height, flags, internal_format);
 
   _cogl_texture_driver_gen (GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
-  GE( glBindTexture (GL_TEXTURE_2D, tex_2d->gl_texture) );
+  _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
+                                   tex_2d->gl_texture,
+                                   FALSE);
   GE( glTexImage2D (GL_TEXTURE_2D, 0, gl_intformat,
                     width, height, 0, gl_format, gl_type, NULL) );
 
@@ -261,57 +254,80 @@ _cogl_texture_2d_new_with_size (unsigned int     width,
 }
 
 CoglHandle
-_cogl_texture_2d_new_from_bitmap (CoglHandle       bmp_handle,
+_cogl_texture_2d_new_from_bitmap (CoglBitmap      *bmp,
                                   CoglTextureFlags flags,
                                   CoglPixelFormat  internal_format)
 {
   CoglTexture2D *tex_2d;
-  CoglBitmap    *bmp = (CoglBitmap *)bmp_handle;
-  CoglBitmap     dst_bmp;
-  gboolean       dst_bmp_owner;
+  CoglBitmap    *dst_bmp;
   GLenum         gl_intformat;
   GLenum         gl_format;
   GLenum         gl_type;
+  guint8        *data;
 
-  g_return_val_if_fail (bmp_handle != COGL_INVALID_HANDLE, COGL_INVALID_HANDLE);
+  g_return_val_if_fail (bmp != NULL, COGL_INVALID_HANDLE);
 
-  internal_format = _cogl_texture_determine_internal_format (bmp->format,
-                                                             internal_format);
+  internal_format =
+    _cogl_texture_determine_internal_format (_cogl_bitmap_get_format (bmp),
+                                             internal_format);
 
-  if (!_cogl_texture_2d_can_create (bmp->width, bmp->height, internal_format))
+  if (!_cogl_texture_2d_can_create (_cogl_bitmap_get_width (bmp),
+                                    _cogl_bitmap_get_height (bmp),
+                                    internal_format))
     return COGL_INVALID_HANDLE;
 
-  if (!_cogl_texture_prepare_for_upload (bmp,
-                                         internal_format,
-                                         &internal_format,
-                                         &dst_bmp,
-                                         &dst_bmp_owner,
-                                         &gl_intformat,
-                                         &gl_format,
-                                         &gl_type))
+  if ((dst_bmp = _cogl_texture_prepare_for_upload (bmp,
+                                                   internal_format,
+                                                   &internal_format,
+                                                   &gl_intformat,
+                                                   &gl_format,
+                                                   &gl_type)) == NULL)
     return COGL_INVALID_HANDLE;
 
-  tex_2d = _cogl_texture_2d_create_base (bmp->width,
-                                         bmp->height,
+  tex_2d = _cogl_texture_2d_create_base (_cogl_bitmap_get_width (bmp),
+                                         _cogl_bitmap_get_height (bmp),
                                          flags,
                                          internal_format);
 
+  /* Keep a copy of the first pixel so that if glGenerateMipmap isn't
+     supported we can fallback to using GL_GENERATE_MIPMAP */
+  if (!cogl_features_available (COGL_FEATURE_OFFSCREEN) &&
+      (data = _cogl_bitmap_map (dst_bmp,
+                                COGL_BUFFER_ACCESS_READ, 0)))
+    {
+      tex_2d->first_pixel.gl_format = gl_format;
+      tex_2d->first_pixel.gl_type = gl_type;
+      memcpy (tex_2d->first_pixel.data, data,
+              _cogl_get_format_bpp (_cogl_bitmap_get_format (dst_bmp)));
+
+      _cogl_bitmap_unmap (dst_bmp);
+    }
+
   _cogl_texture_driver_gen (GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
   _cogl_texture_driver_upload_to_gl (GL_TEXTURE_2D,
                                      tex_2d->gl_texture,
-                                     &dst_bmp,
+                                     FALSE,
+                                     dst_bmp,
                                      gl_intformat,
                                      gl_format,
                                      gl_type);
 
   tex_2d->gl_format = gl_intformat;
 
-  if (dst_bmp_owner)
-    g_free (dst_bmp.data);
+  cogl_object_unref (dst_bmp);
 
   return _cogl_texture_2d_handle_new (tex_2d);
 }
 
+void
+_cogl_texture_2d_externally_modified (CoglHandle handle)
+{
+  if (!_cogl_is_texture_2d (handle))
+    return;
+
+  COGL_TEXTURE_2D (handle)->mipmaps_dirty = TRUE;
+}
+
 static int
 _cogl_texture_2d_get_max_waste (CoglTexture *tex)
 {
@@ -389,29 +405,49 @@ _cogl_texture_2d_set_filters (CoglTexture *tex,
   tex_2d->mag_filter = mag_filter;
 
   /* Apply new filters to the texture */
-  GE( glBindTexture (GL_TEXTURE_2D, tex_2d->gl_texture) );
+  _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
+                                   tex_2d->gl_texture,
+                                   FALSE);
   GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, mag_filter) );
   GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, min_filter) );
 }
 
 static void
-_cogl_texture_2d_ensure_mipmaps (CoglTexture *tex)
+_cogl_texture_2d_pre_paint (CoglTexture *tex, CoglTexturePrePaintFlags flags)
 {
   CoglTexture2D *tex_2d = COGL_TEXTURE_2D (tex);
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   /* Only update if the mipmaps are dirty */
-  if (!tex_2d->auto_mipmap || !tex_2d->mipmaps_dirty)
-    return;
+  if ((flags & COGL_TEXTURE_NEEDS_MIPMAP) &&
+      tex_2d->auto_mipmap && tex_2d->mipmaps_dirty)
+    {
+      _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
+                                       tex_2d->gl_texture,
+                                       FALSE);
 
-  GE( glBindTexture (GL_TEXTURE_2D, tex_2d->gl_texture) );
-  /* glGenerateMipmap is defined in the FBO extension. We only allow
-     CoglTexture2D instances to be created if this feature is
-     available so we don't need to check for the extension */
-  _cogl_texture_driver_gl_generate_mipmaps (GL_TEXTURE_2D);
+      /* glGenerateMipmap is defined in the FBO extension. If it's not
+         available we'll fallback to temporarily enabling
+         GL_GENERATE_MIPMAP and reuploading the first pixel */
+      if (cogl_features_available (COGL_FEATURE_OFFSCREEN))
+        _cogl_texture_driver_gl_generate_mipmaps (GL_TEXTURE_2D);
+      else
+        {
+          GE( glTexParameteri (GL_TEXTURE_2D,
+                               GL_GENERATE_MIPMAP,
+                               GL_TRUE) );
+          GE( glTexSubImage2D (GL_TEXTURE_2D, 0, 0, 0, 1, 1,
+                               tex_2d->first_pixel.gl_format,
+                               tex_2d->first_pixel.gl_type,
+                               tex_2d->first_pixel.data) );
+          GE( glTexParameteri (GL_TEXTURE_2D,
+                               GL_GENERATE_MIPMAP,
+                               GL_FALSE) );
+        }
 
-  tex_2d->mipmaps_dirty = FALSE;
+      tex_2d->mipmaps_dirty = FALSE;
+    }
 }
 
 static void
@@ -428,69 +464,51 @@ _cogl_texture_2d_set_region (CoglTexture    *tex,
                              int             dst_y,
                              unsigned int    dst_width,
                              unsigned int    dst_height,
-                             int             width,
-                             int             height,
-                             CoglPixelFormat format,
-                             unsigned int    rowstride,
-                             const guint8   *data)
+                             CoglBitmap     *bmp)
 {
-  CoglTexture2D   *tex_2d = COGL_TEXTURE_2D (tex);
-  int              bpp;
-  CoglBitmap       source_bmp;
-  CoglBitmap       tmp_bmp;
-  gboolean         tmp_bmp_owner = FALSE;
-  GLenum           closest_gl_format;
-  GLenum           closest_gl_type;
+  CoglTexture2D *tex_2d = COGL_TEXTURE_2D (tex);
+  GLenum         gl_format;
+  GLenum         gl_type;
+  guint8        *data;
 
-  /* Check for valid format */
-  if (format == COGL_PIXEL_FORMAT_ANY)
-    return FALSE;
+  _cogl_pixel_format_to_gl (_cogl_bitmap_get_format (bmp),
+                            NULL, /* internal format */
+                            &gl_format,
+                            &gl_type);
 
-  /* Shortcut out early if the image is empty */
-  if (width == 0 || height == 0)
-    return TRUE;
+  /* If this touches the first pixel then we'll update our copy */
+  if (dst_x == 0 && dst_y == 0 &&
+      !cogl_features_available (COGL_FEATURE_OFFSCREEN) &&
+      (data = _cogl_bitmap_map (bmp, COGL_BUFFER_ACCESS_READ, 0)))
+    {
+      CoglPixelFormat bpp =
+        _cogl_get_format_bpp (_cogl_bitmap_get_format (bmp));
+      tex_2d->first_pixel.gl_format = gl_format;
+      tex_2d->first_pixel.gl_type = gl_type;
+      memcpy (tex_2d->first_pixel.data,
+              data + _cogl_bitmap_get_rowstride (bmp) * src_y + bpp * src_x,
+              bpp);
 
-  /* Init source bitmap */
-  source_bmp.width = width;
-  source_bmp.height = height;
-  source_bmp.format = format;
-  source_bmp.data = (guint8 *)data;
-
-  /* Rowstride from width if none specified */
-  bpp = _cogl_get_format_bpp (format);
-  source_bmp.rowstride = (rowstride == 0) ? width * bpp : rowstride;
-
-  /* Prepare the bitmap so that it will do the premultiplication
-     conversion */
-  _cogl_texture_prepare_for_upload (&source_bmp,
-                                    tex_2d->format,
-                                    NULL,
-                                    &tmp_bmp,
-                                    &tmp_bmp_owner,
-                                    NULL,
-                                    &closest_gl_format,
-                                    &closest_gl_type);
+      _cogl_bitmap_unmap (bmp);
+    }
 
   /* Send data to GL */
   _cogl_texture_driver_upload_subregion_to_gl (GL_TEXTURE_2D,
                                                tex_2d->gl_texture,
+                                               FALSE,
                                                src_x, src_y,
                                                dst_x, dst_y,
                                                dst_width, dst_height,
-                                               &tmp_bmp,
-                                               closest_gl_format,
-                                               closest_gl_type);
-
-  /* Free data if owner */
-  if (tmp_bmp_owner)
-    g_free (tmp_bmp.data);
+                                               bmp,
+                                               gl_format,
+                                               gl_type);
 
   tex_2d->mipmaps_dirty = TRUE;
 
   return TRUE;
 }
 
-static int
+static gboolean
 _cogl_texture_2d_get_data (CoglTexture     *tex,
                            CoglPixelFormat  format,
                            unsigned int     rowstride,
@@ -498,101 +516,25 @@ _cogl_texture_2d_get_data (CoglTexture     *tex,
 {
   CoglTexture2D   *tex_2d = COGL_TEXTURE_2D (tex);
   int              bpp;
-  int              byte_size;
-  CoglPixelFormat  closest_format;
-  int              closest_bpp;
-  GLenum           closest_gl_format;
-  GLenum           closest_gl_type;
-  CoglBitmap       target_bmp;
-  CoglBitmap       new_bmp;
-  gboolean         success;
-  guint8          *src;
-  guint8          *dst;
-  int              y;
+  GLenum           gl_format;
+  GLenum           gl_type;
 
-  /* Default to internal format if none specified */
-  if (format == COGL_PIXEL_FORMAT_ANY)
-    format = tex_2d->format;
-
-  /* Rowstride from texture width if none specified */
   bpp = _cogl_get_format_bpp (format);
-  if (rowstride == 0)
-    rowstride = tex_2d->width * bpp;
 
-  /* Return byte size if only that requested */
-  byte_size =  tex_2d->height * rowstride;
-  if (data == NULL)
-    return byte_size;
+  _cogl_pixel_format_to_gl (format,
+                            NULL, /* internal format */
+                            &gl_format,
+                            &gl_type);
 
-  closest_format =
-    _cogl_texture_driver_find_best_gl_get_data_format (format,
-                                                       &closest_gl_format,
-                                                       &closest_gl_type);
-  closest_bpp = _cogl_get_format_bpp (closest_format);
+  _cogl_texture_driver_prep_gl_for_pixels_download (rowstride, bpp);
 
-  target_bmp.width = tex_2d->width;
-  target_bmp.height = tex_2d->height;
-
-  /* Is the requested format supported? */
-  if (closest_format == format)
-    {
-      /* Target user data directly */
-      target_bmp.format = format;
-      target_bmp.rowstride = rowstride;
-      target_bmp.data = data;
-    }
-  else
-    {
-      /* Target intermediate buffer */
-      target_bmp.format = closest_format;
-      target_bmp.rowstride = target_bmp.width * closest_bpp;
-      target_bmp.data = g_malloc (target_bmp.height * target_bmp.rowstride);
-    }
-
-  _cogl_texture_driver_prep_gl_for_pixels_download (target_bmp.rowstride,
-                                                    closest_bpp);
-
-  GE( glBindTexture (GL_TEXTURE_2D, tex_2d->gl_texture) );
-  if (!_cogl_texture_driver_gl_get_tex_image (GL_TEXTURE_2D,
-                                              closest_gl_format,
-                                              closest_gl_type,
-                                              target_bmp.data))
-    {
-      /* XXX: In some cases _cogl_texture_2d_download_from_gl may
-       * fail to read back the texture data; such as for GLES which doesn't
-       * support glGetTexImage, so here we fallback to drawing the texture
-       * and reading the pixels from the framebuffer. */
-      _cogl_texture_draw_and_read (tex, &target_bmp,
-                                   closest_gl_format,
-                                   closest_gl_type);
-    }
-
-  /* Was intermediate used? */
-  if (closest_format != format)
-    {
-      /* Convert to requested format */
-      success = _cogl_bitmap_convert_format_and_premult (&target_bmp,
-                                                         &new_bmp,
-                                                         format);
-
-      /* Free intermediate data and return if failed */
-      g_free (target_bmp.data);
-      if (!success)
-        return 0;
-
-      /* Copy to user buffer */
-      for (y = 0; y < new_bmp.height; ++y)
-        {
-          src = new_bmp.data + y * new_bmp.rowstride;
-          dst = data + y * rowstride;
-          memcpy (dst, src, new_bmp.width);
-        }
-
-      /* Free converted data */
-      g_free (new_bmp.data);
-    }
-
-  return byte_size;
+  _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
+                                   tex_2d->gl_texture,
+                                   FALSE);
+  return _cogl_texture_driver_gl_get_tex_image (GL_TEXTURE_2D,
+                                                gl_format,
+                                                gl_type,
+                                                data);
 }
 
 static CoglPixelFormat
@@ -632,11 +574,12 @@ cogl_texture_2d_vtable =
     _cogl_texture_2d_transform_quad_coords_to_gl,
     _cogl_texture_2d_get_gl_texture,
     _cogl_texture_2d_set_filters,
-    _cogl_texture_2d_ensure_mipmaps,
+    _cogl_texture_2d_pre_paint,
     _cogl_texture_2d_ensure_non_quad_rendering,
     _cogl_texture_2d_set_wrap_mode_parameters,
     _cogl_texture_2d_get_format,
     _cogl_texture_2d_get_gl_format,
     _cogl_texture_2d_get_width,
-    _cogl_texture_2d_get_height
+    _cogl_texture_2d_get_height,
+    NULL /* is_foreign */
   };
