@@ -30,6 +30,7 @@
 
 #include "clutter-stage-x11.h"
 #include "clutter-backend-x11.h"
+#include "clutter-keymap-x11.h"
 #include "clutter-x11.h"
 
 #include "../clutter-backend.h"
@@ -38,6 +39,8 @@
 #include "../clutter-debug.h"
 #include "../clutter-main.h"
 
+#include "cogl/cogl-internal.h"
+
 #include <string.h>
 
 #include <glib.h>
@@ -52,6 +55,10 @@
 #include <X11/extensions/XInput.h>
 #endif
 
+#ifdef HAVE_XKB
+#include <X11/XKBlib.h>
+#endif
+
 /* XEMBED protocol support for toolkit embedding */
 #define XEMBED_MAPPED                   (1 << 0)
 #define MAX_SUPPORTED_XEMBED_VERSION    1
@@ -83,6 +90,38 @@ struct _ClutterEventSource
   GPollFD event_poll_fd;
 };
 
+struct _ClutterEventX11
+{
+  /* additional fields for Key events */
+  gint key_group;
+
+  guint key_is_modifier : 1;
+  guint num_lock_set    : 1;
+  guint caps_lock_set   : 1;
+};
+
+ClutterEventX11 *
+_clutter_event_x11_new (void)
+{
+  return g_slice_new0 (ClutterEventX11);
+}
+
+ClutterEventX11 *
+_clutter_event_x11_copy (ClutterEventX11 *event_x11)
+{
+  if (event_x11 != NULL)
+    return g_slice_dup (ClutterEventX11, event_x11);
+
+  return NULL;
+}
+
+void
+_clutter_event_x11_free (ClutterEventX11 *event_x11)
+{
+  if (event_x11 != NULL)
+    g_slice_free (ClutterEventX11, event_x11);
+}
+
 static gboolean clutter_event_prepare  (GSource     *source,
                                         gint        *timeout);
 static gboolean clutter_event_check    (GSource     *source);
@@ -105,6 +144,10 @@ clutter_event_source_new (ClutterBackend *backend)
   GSource *source = g_source_new (&event_funcs, sizeof (ClutterEventSource));
   ClutterEventSource *event_source = (ClutterEventSource *) source;
 
+#if GLIB_CHECK_VERSION (2, 25, 8)
+  g_source_set_name (source, "Clutter X11 Event");
+#endif
+
   event_source->backend = backend;
 
   return source;
@@ -289,16 +332,18 @@ convert_xdevicekey_to_xkey (XDeviceKeyEvent *xkev,
 }
 #endif /* HAVE_XINPUT */
 
-static void
-translate_key_event (ClutterBackend   *backend,
-                     ClutterEvent     *event,
-                     XEvent           *xevent)
+static inline void
+translate_key_event (ClutterBackendX11 *backend_x11,
+                     ClutterEvent      *event,
+                     XEvent            *xevent)
 {
-  char buffer[256+1];
+  ClutterEventX11 *event_x11;
+  char buffer[256 + 1];
   int n;
-  
-  CLUTTER_NOTE (EVENT, "Translating key %s event",
-                xevent->xany.type == KeyPress ? "press" : "release");
+
+  /* KeyEvents have platform specific data associated to them */
+  event_x11 = _clutter_event_x11_new ();
+  _clutter_event_set_platform_data (event, event_x11);
 
   event->key.time = xevent->xkey.time;
   event->key.modifier_state = (ClutterModifierType) xevent->xkey.state;
@@ -306,9 +351,21 @@ translate_key_event (ClutterBackend   *backend,
 
   /* keyval is the key ignoring all modifiers ('1' vs. '!') */
   event->key.keyval =
-    XKeycodeToKeysym (xevent->xkey.display,
-                      xevent->xkey.keycode,
-                      0);
+    _clutter_keymap_x11_translate_key_state (backend_x11->keymap,
+                                             event->key.hardware_keycode,
+                                             event->key.modifier_state,
+                                             NULL);
+
+  event_x11->key_group =
+    _clutter_keymap_x11_get_key_group (backend_x11->keymap,
+                                       event->key.modifier_state);
+  event_x11->key_is_modifier =
+    _clutter_keymap_x11_get_is_modifier (backend_x11->keymap,
+                                         event->key.hardware_keycode);
+  event_x11->num_lock_set =
+    _clutter_keymap_x11_get_num_lock_state (backend_x11->keymap);
+  event_x11->caps_lock_set =
+    _clutter_keymap_x11_get_caps_lock_state (backend_x11->keymap);
 
   /* unicode_value is the printable representation */
   n = XLookupString (&xevent->xkey, buffer, sizeof (buffer) - 1, NULL, NULL);
@@ -318,10 +375,13 @@ translate_key_event (ClutterBackend   *backend,
       event->key.unicode_value = g_utf8_get_char_validated (buffer, n);
       if ((event->key.unicode_value != -1) &&
           (event->key.unicode_value != -2))
-        return;
+        goto out;
     }
-  
-  event->key.unicode_value = (gunichar)'\0';
+  else
+    event->key.unicode_value = (gunichar)'\0';
+
+out:
+  return;
 }
 
 static gboolean
@@ -454,6 +514,10 @@ event_translate (ClutterBackend *backend,
         }
     }
 
+  /* Pass the event through Cogl */
+  if (_cogl_xlib_handle_event (xevent) == COGL_XLIB_FILTER_REMOVE)
+    return FALSE;
+
   /* Do further processing only on events for the stage window (the x11
    * filters might be getting events for other windows, so do not mess
    * them about.
@@ -499,7 +563,9 @@ event_translate (ClutterBackend *backend,
           if ((stage_x11->state & CLUTTER_STAGE_STATE_FULLSCREEN) ||
               (stage_x11->xwin_width != xevent->xconfigure.width) ||
               (stage_x11->xwin_height != xevent->xconfigure.height))
-          clutter_actor_queue_relayout (CLUTTER_ACTOR (stage));
+            {
+              clutter_actor_queue_relayout (CLUTTER_ACTOR (stage));
+            }
 
           /* If we're fullscreened, we want these variables to
            * represent the size of the window before it was set
@@ -516,7 +582,7 @@ event_translate (ClutterBackend *backend,
                                   xevent->xconfigure.height);
 
           CLUTTER_UNSET_PRIVATE_FLAGS (stage_x11->wrapper,
-                                       CLUTTER_STAGE_IN_RESIZE);
+                                       CLUTTER_IN_RESIZE);
 
           /* the resize process is complete, so we can ask the stage
            * to set up the GL viewport with the new size
@@ -659,7 +725,7 @@ event_translate (ClutterBackend *backend,
         clutter_device_manager_get_core_device (manager,
                                                 CLUTTER_KEYBOARD_DEVICE);
 
-      translate_key_event (backend, event, xevent);
+      translate_key_event (backend_x11, event, xevent);
 
       set_user_time (backend_x11, &xwindow, xevent->xkey.time);
       break;
@@ -675,8 +741,11 @@ event_translate (ClutterBackend *backend,
        * the next event and check if it's a KeyPress for the same key
        * and timestamp - and then ignore it if it matches the
        * KeyRelease
+       *
+       * if we have XKB, and autorepeat is enabled, then this becomes
+       * a no-op
        */
-      if (XPending (xevent->xkey.display))
+      if (!backend_x11->have_xkb_autorepeat && XPending (xevent->xkey.display))
         {
           XEvent next_event;
 
@@ -696,7 +765,7 @@ event_translate (ClutterBackend *backend,
         clutter_device_manager_get_core_device (manager,
                                                 CLUTTER_KEYBOARD_DEVICE);
 
-      translate_key_event (backend, event, xevent);
+      translate_key_event (backend_x11, event, xevent);
       break;
 
     default:
@@ -976,7 +1045,7 @@ event_translate (ClutterBackend *backend,
                                           ? CLUTTER_KEY_PRESS
                                           : CLUTTER_KEY_RELEASE;
 
-          translate_key_event (backend, event, &xevent_converted);
+          translate_key_event (backend_x11, event, &xevent_converted);
 
           if (xevent->type == key_press)
             set_user_time (backend_x11, &xwindow, xkev->time);
@@ -1021,9 +1090,7 @@ events_queue (ClutterBackend *backend)
 	  g_queue_push_head (clutter_context->events_queue, event);
         }
       else
-        {
-          clutter_event_free (event);
-        }
+        clutter_event_free (event);
     }
 }
 
@@ -1032,7 +1099,11 @@ events_queue (ClutterBackend *backend)
  * @xevent: pointer to XEvent structure
  *
  * This function processes a single X event; it can be used to hook
- * into external X event retrieval (for example that done by GDK).
+ * into external X11 event processing (for example, a GDK filter
+ * function).
+ *
+ * If clutter_x11_disable_event_retrieval() has been called, you must
+ * let this function process events to update Clutter's internal state.
  *
  * Return value: #ClutterX11FilterReturn. %CLUTTER_X11_FILTER_REMOVE
  *  indicates that Clutter has internally handled the event and the
@@ -1042,7 +1113,7 @@ events_queue (ClutterBackend *backend)
  *  any exclusive action. %CLUTTER_X11_FILTER_TRANSLATE will not
  *  occur.
  *
- * Since:  0.8
+ * Since: 0.8
  */
 ClutterX11FilterReturn
 clutter_x11_handle_event (XEvent *xevent)
@@ -1199,3 +1270,29 @@ clutter_x11_get_current_event_time (void)
 
   return CLUTTER_BACKEND_X11 (backend)->last_event_time;
 }
+
+/**
+ * clutter_x11_event_get_key_group:
+ * @event: a #ClutterEvent of type %CLUTTER_KEY_PRESS or %CLUTTER_KEY_RELEASE
+ *
+ * Retrieves the group for the modifiers set in @event
+ *
+ * Return value: the group id
+ *
+ * Since: 1.4
+ */
+gint
+clutter_x11_event_get_key_group (const ClutterEvent *event)
+{
+  ClutterEventX11 *event_x11;
+
+  g_return_val_if_fail (event != NULL, 0);
+  g_return_val_if_fail (event->type == CLUTTER_KEY_PRESS ||
+                        event->type == CLUTTER_KEY_RELEASE, 0);
+
+  event_x11 = _clutter_event_get_platform_data (event);
+  if (event_x11 == NULL)
+    return 0;
+
+  return event_x11->key_group;
+}
